import { Plugin } from 'ckeditor5/src/core'
import { Widget, toWidget } from 'ckeditor5/src/widget'
import type { DowncastWriter, Element } from '@ckeditor/ckeditor5-engine'
import InsertIconifyFieldCommand from './insert_iconify_field_command'

export default class IconifyFieldEditing extends Plugin {
  static get requires() {
    return [Widget]
  }

  init() {
    this._defineSchema()
    this._defineConverters()

    this.editor.commands.add(
      'insertIconifyField',
      new InsertIconifyFieldCommand(this.editor),
    )
  }

  _defineSchema() {
    const { schema } = this.editor.model

    schema.register('iconifyField', {
      inheritAllFrom: '$inlineObject',
      allowAttributes: [
        'data-icon',
      ],
    })
  }

  _defineConverters() {
    const { conversion } = this.editor

    conversion.for('upcast').elementToElement({
      // model: 'iconifyField',
      view: {
        name: 'span',
        classes: 'iconify-field',
      },
      model: (viewElement, { writer }) => {
        return writer.createElement('iconifyField', {
          'data-icon': viewElement.getAttribute('data-icon'),
        })
      },
    })

    conversion.for('dataDowncast').elementToElement({
      model: 'iconifyField',
      view: (modelElement, { writer }) => {
        return this.createIconifyFieldView(modelElement, writer)
      },
    })

    conversion.for('editingDowncast').elementToElement({
      model: 'iconifyField',
      view: (modelElement, { writer }) => {
        const element = this.createIconifyFieldView(modelElement, writer)

        return toWidget(element, writer, { label: 'Icon' })
      },
    })
  }

  createIconifyFieldView(modelElement: Element, writer: DowncastWriter) {
    const icon = modelElement.getAttribute('data-icon') ?? ''

    const element = writer.createContainerElement(
      'span',
      {
        'class': 'iconify-field',
        'data-icon': icon,
        'style': icon ? `background-image: url(/api/iconify_field_ckeditor/icon/${icon})` : undefined,
      },
    )

    return element
  }
}
