<?php

declare(strict_types=1);

namespace Drupal\iconify_field\Service;

use Drupal\Component\Utility\Html;
use Drupal\Core\Render\Markup;
use Iconify\IconsJSON\Finder;

/**
 * {@inheritdoc}
 */
class IconResolver implements IconResolverInterface {

  /**
   * The Iconify icon finder.
   *
   * @var \Iconify\IconsJSON\Finder
   */
  protected $finder;

  /**
   * IconResolver constructor.
   */
  public function __construct() {
    $this->finder = new Finder();
  }

  /**
   * {@inheritdoc}
   */
  public function getIcon(?string $icon_name, bool $autosize = FALSE) : Markup {
    $fallback = Markup::create(
      sprintf(
        '<span class="iconify">%s</span>',

        // XSS is bad, m'kay?
        Html::escape($icon_name ?? ''),
      )
    );

    if (!$icon_name || strpos($icon_name, ':') === FALSE) {
      return $fallback;
    }

    [$collection_name, $icon_name] = explode(':', $icon_name);

    $collection_path = Finder::locate($collection_name);

    if (!file_exists($collection_path)) {
      return $fallback;
    }

    $collection = json_decode(file_get_contents($collection_path), TRUE);

    if (!json_last_error() === JSON_ERROR_NONE || !isset($collection['icons'][$icon_name])) {
      return $fallback;
    }

    $collection_width = $collection['width'];
    $collection_height = $collection['height'];
    $icon_width = $collection['icons'][$icon_name]['width'] ?? $collection_width;
    $normalized_width = round(($icon_width / $collection_width) * 100) / 100 . 'em';

    $xmlns = 'xmlns="http://www.w3.org/2000/svg"';
    $style = 'style="color-scheme: light dark;"';

    $svg_prefix = sprintf(
      '<svg %s class="iconify" %s width="%s" height="%s" viewbox="0 0 %s %s">',
      $xmlns,
      $style,
      $autosize ? $icon_width : $normalized_width,
      $autosize ? $collection_height : '1em',
      $icon_width,
      $collection_height,
    );

    return Markup::create(
      $svg_prefix .
      $collection['icons'][$icon_name]['body'] .
      '</svg>'
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getIconifyCollections(): array {
    $collections = [];

    $collections_data = Finder::collections();

    foreach ($collections_data as $collection_id => $collection) {
      $collection_name = sprintf('%s - %s', $collection['name'], $collection['license']['title']);
      $collections[$collection['category'] ?? 'Misc'][$collection_id] = $collection_name;
    }

    return $collections;
  }

}
