<?php

declare(strict_types=1);

namespace Drupal\iconify_field\Service;

use Drupal\Core\Render\Markup;

/**
 * Resolves icons and collections from the Iconify library.
 *
 * @package Drupal\iconify_field\Service
 */
interface IconResolverInterface {

  /**
   * Get an SVG from an iconify field name.
   *
   * @param string $icon_name
   *   The icon name to get the icon from.
   * @param bool $autosize
   *   Whether to autosize the icon or keep it at 1em.
   *
   * @return \Drupal\Core\Render\MarkupInterface
   *   The SVG icon.
   */
  public function getIcon(?string $icon_name, bool $autosize = FALSE): Markup;

  /**
   * Get a list of available Iconify collections.
   *
   * @return array
   *   An array of available collections.
   */
  public function getIconifyCollections(): array;

}
